-- SLDataText Statline Copyright (c) 2008, 2009 Jeff "Taffu" Fancher <jdfancher@gmail.com> All rights reserved.
-- Version: 1.1.1 -- Rev Date: 05/31/2009

local SLDataText = LibStub("AceAddon-3.0"):GetAddon("SLDataText")
local L = LibStub("AceLocale-3.0"):GetLocale("SLDataText")
local media = LibStub("LibSharedMedia-3.0")
local justTable, strataTable, pointTable = SLDataText.just, SLDataText.strata, SLDataText.point
local db

local MODNAME = "Statline"
local Statline = SLDataText:NewModule(MODNAME, "AceEvent-3.0")

local optGetter, optSetter
do
	function optGetter(info)
		local key = info[#info]
		return db[key]
	end

	function optSetter(info, value)
		local key = info[#info]
		db[key] = value
        SLDataText:RefreshModule(Statline)
	end
end

local statistics = {
    ["Armor Penetration"] = L["Armor Penetration"],
    ["Armor"] = L["Armor"],
    ["Attack Power"] = L["Attack Power"],
    ["Attack Speed"] = L["Attack Speed"],
    ["Defense"] = L["Defense"],
    ["Ranged Attack Power"] = L["Ranged Attack Power"],
    ["Spell Penetration"] = L["Spell Penetration"],
    ["Spell Damage"] = L["Spell Damage"],
    ["Spell Healing"] = L["Spell Healing"],
    ["Mana Regen"] = L["Mana Regen"],
    ["Parry Chance"] = L["Parry Chance"],
    ["Dodge Chance"] = L["Dodge Chance"],
    ["Melee Crit"] = L["Melee Crit"],
    ["Ranged Crit"] = L["Ranged Crit"],
    ["Spell Crit"] = L["Spell Crit"],
    ["Hit Rating"] = L["Hit Rating"],
    ["Block Chance"] = L["Block Chance"],
    ["Expertise"] = L["Expertise"],
    ["Haste Rating"] = L["Haste Rating"],
    ["Resilience"] = L["Resilience"],
}

local options
local function getOptions()
	if not options then options = {
        type = "group",
        name = L["Statline"],
        arg = MODNAME,
        get = optGetter,
        set = optSetter,
        args = {
            genHeader = {
                type = "header",
                name = L["GenSLSet"],
                order = 25,
            },
            enabled = {
                type = "toggle",
                name = L["Enabled"],
                desc = L["EnabledDesc"],
                get = function() return SLDataText:GetModuleEnabled(MODNAME) end,
                set = function(info, value) 
                    SLDataText:SetModuleEnabled(MODNAME, value)
                    if ( SLDataText:GetModuleEnabled(MODNAME) ) then
                        MODNAME:PLAYER_ENTERING_WORLD()
                    end
                end,
                order = 50,
            },
            hideTooltip = {
                type = "toggle",
                name = L["HideTT"],
                desc = L["HideTTDesc"],
                order = 100,
            },
            noCombatHide = {
                type = "toggle",
                name = L["SIC"],
                desc = L["SICDesc"],
                order = 150,
            },
            dispHeader = {
                type = "header",
                name = L["DispSet"],
                order = 200,
            },
            stat1 = {
                type = "select",
                name = L["Stat1"],
                desc = L["Stat1Desc"],
                values = statistics,
                get = function()
                    local stat1
                    if ( GetActiveTalentGroup() == 1 ) then
                        stat1 = db.primary.stat1
                    elseif ( GetActiveTalentGroup() == 2 ) then
                        stat1 = db.secondary.stat1
                    end
                    return stat1
                end,
                set = function(_, value)
                    if ( GetActiveTalentGroup() == 1 ) then
                        db.primary.stat1 = value
                    elseif ( GetActiveTalentGroup() == 2 ) then
                        db.secondary.stat1 = value
                    end
                    Statline:Refresh()
                end,
                width = "double",
                order = 220,
            },
            stat2 = {
                type = "select",
                name = L["Stat2"],
                desc = L["Stat2Desc"],
                values = statistics,
                get = function()
                    local stat1
                    if ( GetActiveTalentGroup() == 1 ) then
                        stat1 = db.primary.stat2
                    elseif ( GetActiveTalentGroup() == 2 ) then
                        stat1 = db.secondary.stat2
                    end
                    return stat1
                end,
                set = function(_, value)
                    if ( GetActiveTalentGroup() == 1 ) then
                        db.primary.stat2 = value
                    elseif ( GetActiveTalentGroup() == 2 ) then
                        db.secondary.stat2 = value
                    end
                    Statline:Refresh()
                end,
                width = "double",
                order = 240,
            },
            stat3 = {
                type = "select",
                name = L["Stat3"],
                desc = L["Stat3Desc"],
                values = statistics,
                get = function()
                    local stat1
                    if ( GetActiveTalentGroup() == 1 ) then
                        stat1 = db.primary.stat3
                    elseif ( GetActiveTalentGroup() == 2 ) then
                        stat1 = db.secondary.stat3
                    end
                    return stat1
                end,
                set = function(_, value)
                    if ( GetActiveTalentGroup() == 1 ) then
                        db.primary.stat3 = value
                    elseif ( GetActiveTalentGroup() == 2 ) then
                        db.secondary.stat3 = value
                    end
                    Statline:Refresh()
                end,
                width = "double",
                order = 260,
            },
            useGlobalFont = {
                type = "toggle",
                name = L["UseGblFont"],
                desc = L["UseGblFontDesc"],
                order = 300,
            },
            useGlobalFontSize = {
                type = "toggle",
                name = L["UseGblFSize"],
                desc = L["UseGblFSizeDesc"],
                order = 350,
            },
            fontFace = {
                type = "select",
                name = L["Font"],
                desc = L["FontDesc"],
                disabled = function()
                    local isTrue
                    if ( db.useGlobalFont ) then isTrue = true else isTrue = false end
                    return isTrue
                end,
                values = media:List("font"),
                get = function()
                    for k, v in pairs(media:List("font")) do
                        if db.fontFace == v then
                            return k
                        end
                    end
                end,
                set = function(_, font)
                    local list = media:List("font")
                    db.fontFace = list[font]
                    SLDataText:RefreshModule(Statline)
                end,
                width = "double",
                order = 600,
            },
            fontSize = {
                type = "range",
                name = L["FontSize"],
                desc = L["FontSizeDesc"],
                disabled = function()
                    local isTrue
                    if ( db.useGlobalFontSize ) then isTrue = true else isTrue = false end
                    return isTrue
                end,
                min = 6, max = 36, step = 1,
                width = "double",
                order = 650,
            },
            posHeader = {
                type = "header",
                name = L["LaySet"],
                order = 700,
            },
            justify = {
                type = "select",
                name = L["TextJust"],
                desc = L["TextJustDesc"],
                values = justTable,
                width = "double",
                order = 750,
            },
            anchor = {
                type = "input",
                name = L["ParFrm"],
                desc = L["ParFrmDesc"],
                get = function() return db.anchor end,
                width = "double",
                order = 800,
            },
            anchorFrom = {
                type = "select",
                name = L["AnchFrom"],
                desc = L["AnchFromDesc"],
                values = pointTable,
                get = function() return db.anchorFrom end,
                width = "double",
                order = 900,
            },
            offX = {
                type = "input",
                name = L["XOff"],
                desc = L["XOffDesc"],
                get = function() return tostring(db.offX) end,
                width = "double",
                order = 1000,
            },
            offY = {
                type = "input",
                name = L["YOff"],
                desc = L["YOffDesc"],
                get = function() return tostring(db.offY) end,
                width = "double",
                order = 1100,
            },
            strata = {
                type = "select",
                name = L["Strata"],
                desc = L["StrataDesc"],
                values = strataTable,
                width = "double",
                order = 1600,
            },
        },
    }
	end
	
	return options
end

local function updateStatCore()
    
end

local function buildModule(self)
    if ( not self.frame ) then self.frame = CreateFrame("Frame", "SLDT_Statline", UIParent) end -- The frame
    -- Blocks
    if ( not self.block1 ) then self.block1 = CreateFrame("Frame", nil, self.frame) end
    if ( not self.string1 ) then self.string1 = self.block1:CreateFontString(nil, "OVERLAY") end
    if ( not self.block2 ) then self.block2 = CreateFrame("Frame", nil, self.frame) end
    if ( not self.string2 ) then self.string2 = self.block2:CreateFontString(nil, "OVERLAY") end
    if ( not self.block3 ) then self.block3 = CreateFrame("Frame", nil, self.frame) end
    if ( not self.string3 ) then self.string3 = self.block3:CreateFontString(nil, "OVERLAY") end
    
    if ( not self.string ) then self.string = self.frame:CreateFontString(nil, "OVERLAY") end -- The font string (dummy)
    self:RegisterEvent("PLAYER_ENTERING_WORLD")
end

function Statline:PLAYER_ENTERING_WORLD()
    local enabled = SLDataText:GetModuleEnabled(MODNAME)
    if ( enabled ) then
        
    end
    SLDataText:RefreshModule(self)
    self:UnregisterEvent("PLAYER_ENTERING_WORLD")
end

function Statline:OnInitialize()
	self.db = SLDataText.db:RegisterNamespace(MODNAME)
    self.db:RegisterDefaults({
        profile = {
            primary = {
                stat1 = "Spell Healing",
                stat2 = "Mana Regen",
                stat3 = "Spell Crit",
            },
            secondary = {
                stat1 = "Armor",
                stat2 = "Armor Penetration",
                stat3 = "Attack Power",
            },
            hideTooltip = false,
            noCombatHide = false,
            useGlobalFont = true,
            fontFace = "Arial Narrow",
            useGlobalFontSize = true,
            fontSize = 12,
            justify = "CENTER",
            anchorPoint = "CENTER", 
            anchor = "UIParent", 
            anchorFrom = "CENTER", 
            offX = 0, 
            offY = 0,
            strata = "BACKGROUND",
        },
    })
	db = self.db.profile
    
    if ( not self.isMoving ) then self.isMoving = false end
	self:SetEnabledState(SLDataText:GetModuleEnabled(MODNAME))
	SLDataText:RegisterModuleOptions(MODNAME, getOptions)
end

-- Stat functions ---

local function GetArmPen()
    local color = SLDataText:GetColor()
    local armPen = GetArmorPenetration() -- %
    return format("|cff%s%s|r %.1f%%", color, L["ArmPen"], armPen)
end

local function GetBlkChn()
    local color = SLDataText:GetColor()
    local blkChn = GetBlockChance() -- %
    return format("|cff%s%s|r %.1f%%", color, L["Block"], blkChn)
end

local function GetHitRating()
    -- 6, 7, or 8 and 9, 10, 11 are all the same, hit is shared
    local color = SLDataText:GetColor()
    local cmbtRat = GetCombatRating(6)
    return format("|cff%s%s|r %d", color, L["Hit"], cmbtRat)
end

local function GetHasteRating()
    -- 18, 19, 20 are shared Haste Rating
    local color = SLDataText:GetColor()
    local melee = GetCombatRating(18)
    local range = GetCombatRating(19)
    local spell = GetCombatRating(20)
    local minHaste = min(melee, range)
	minHaste = min(minHaste, spell)
    return format("|cff%s%s|r %d", color, L["Haste"], minHaste)
end

local function GetMeleeCrit()
    local color = SLDataText:GetColor()
    local melCrit = GetCritChance() -- %
    return format("|cff%s%s|r %.1f%%", color, L["MelCrit"], melCrit)
end

local function GetSpellCrit()
    -- 1 = Physical, 2 = Holy, 3 = Fire, 4 = Nature, 5 = Frost, 6 = Shadow, 7 = Arcane
    local color = SLDataText:GetColor()
    local minCrit = GetSpellCritChance(2)
    for i = 3, 7 do
        local splCrit = GetSpellCritChance(i)
        minCrit = min(minCrit, splCrit)
    end
    return format("|cff%s%s|r %.1f%%", color, L["SpCrit"], minCrit)
end

local function GetDodgeChn()
    local color = SLDataText:GetColor()
    local dodgeChn = GetDodgeChance() -- %
    return format("|cff%s%s|r %.1f%%", color, L["Dodge"], dodgeChn)
end

local function GetExpPerc()
    local color = SLDataText:GetColor()
    local expPerc = GetExpertisePercent()
    return format("|cff%s%s|r %.1f%%", color, L["Exper"], expPerc)
end

local function GetParChn()
    local color = SLDataText:GetColor()
    local parChn = GetParryChance() -- %
    return format("|cff%s%s|r %.1f%%", color, L["Parry"], parChn)
end

local function GetManaReg()
    local color = SLDataText:GetColor()
    local base, cast = GetManaRegen()
    return format("|cff%s%s|r %d", color, L["MP5"], floor(cast*5))
end

local function GetSplBonusDmg()
    -- 1 = Physical, 2 = Holy, 3 = Fire, 4 = Nature, 5 = Frost, 6 = Shadow, 7 = Arcane
    local color = SLDataText:GetColor()
    local minDmg = GetSpellBonusDamage(2)
    for i = 3, 7 do
        local splDmg = GetSpellBonusDamage(i)
        minDmg = min(minDmg, splDmg)
    end
    return format("|cff%s%s|r %d", color, L["SpDmg"], minDmg)
end

local function GetSplBonusHeal()
    local color = SLDataText:GetColor()
    local bonusHeal = GetSpellBonusHealing()
    return format("|cff%s%s|r %d", color, L["SpHeal"], bonusHeal)
end

local function GetSpellPen()
    local color = SLDataText:GetColor()
    local spellPen = GetSpellPenetration() -- %
    return format("|cff%s%s|r %.1f%%", color, L["SpPen"], spellPen)
end

local function GetArmor()
    local color = SLDataText:GetColor()
    local base, effective, armor, _, _ = UnitArmor("player")
    return format("|cff%s%s|r %d", color, L["Arm"], armor)
end

local function GetAttPwr()
    local color = SLDataText:GetColor()
    local base, pos, neg = UnitAttackPower("player")
    return format("|cff%s%s|r %d", color, L["AttPwr"], ((base+pos)-neg))
end

local function GetRangAttPwr()
    local color = SLDataText:GetColor()
    local base, pos, neg = UnitRangedAttackPower("player")
    return format("|cff%s%s|r %d", color, L["RAttPwr"], ((base+pos)-neg))
end

local function GetRangCrit()
    local color = SLDataText:GetColor()
    local rcrit = GetRangedCritChance()
    return format("|cff%s%s|r %.1f%%", color, L["RaCrit"], rcrit)
end

local function GetAttSpeed()
    local color = SLDataText:GetColor()
    local mainhand, offhand = UnitAttackSpeed("player")
    local speed
    if ( offhand and offhand > 0 ) then
        speed = format("%.1f(%.1f)", mainhand, offhand)
    else
        speed = format("%.1f", mainhand)
    end
    return format("|cff%s%s|r %s", color, L["AttSpd"], speed)
end

local function GetDefense()
    local color = SLDataText:GetColor()
    local base, armor = UnitDefense("player")
    return format("|cff%s%s|r %d", color, L["Def"], base+armor)
end

local function GetResilience()
    local color = SLDataText:GetColor()
    local melee = GetCombatRating(15)
	local ranged = GetCombatRating(16)
	local spell = GetCombatRating(17)
	local minRes = min(melee, ranged)
	minRes = min(minRes, spell)
    return format("|cff%s%s|r %d", color, L["Res"], minRes)
end

-- Stat/Func Tbl ----

local statFunc 
local function RefreshStats()
    if ( statFunc ) then statFunc = nil end
    statFunc = {
        ["Armor Penetration"] = GetArmPen(),
        ["Armor"] = GetArmor(),
        ["Attack Power"] = GetAttPwr(),
        ["Attack Speed"] = GetAttSpeed(),
        ["Defense"] = GetDefense(),
        ["Ranged Attack Power"] = GetRangAttPwr(),
        ["Spell Penetration"] = GetSpellPen(),
        ["Spell Damage"] = GetSplBonusDmg(),
        ["Spell Healing"] = GetSplBonusHeal(),
        ["Mana Regen"] = GetManaReg(),
        ["Parry Chance"] = GetParChn(),
        ["Dodge Chance"] = GetDodgeChn(),
        ["Melee Crit"] = GetMeleeCrit(),
        ["Ranged Crit"] = GetRangCrit(),
        ["Spell Crit"] = GetSpellCrit(),
        ["Hit Rating"] = GetHitRating(),
        ["Block Chance"] = GetBlkChn(),
        ["Expertise"] = GetExpPerc(),
        ["Haste Rating"] = GetHasteRating(),
        ["Resilience"] = GetResilience(),
    }
end

-- Stat Fetch Func --

local function GatherStats(stat1, stat2, stat3)
    RefreshStats()
    return statFunc[stat1], statFunc[stat2], statFunc[stat3]
end

---------------------

function Statline:OnEnable()
    buildModule(self)
    --self:RegisterEvent("UNIT_AURA", "Refresh")
    self:RegisterEvent("UNIT_STATS", "Refresh")
    self:RegisterEvent("UNIT_DAMAGE", "Refresh")
    self:RegisterEvent("UNIT_RANGEDDAMAGE", "Refresh")
    self:RegisterEvent("PLAYER_DAMAGE_DONE_MODS", "Refresh")
    self:RegisterEvent("UNIT_ATTACK_SPEED", "Refresh")
    self:RegisterEvent("UNIT_ATTACK_POWER", "Refresh")
    self:RegisterEvent("UNIT_RANGED_ATTACK_POWER", "Refresh")
    self:RegisterEvent("UNIT_ATTACK", "Refresh")
    self:RegisterEvent("UNIT_RESISTANCES", "Refresh")
    if ( not self.frame:IsShown() ) then self.frame:Show() end
end

function Statline:OnDisable()
    self:UnregisterEvent("UNIT_STATS")
    self:UnregisterEvent("UNIT_DAMAGE")
    self:UnregisterEvent("UNIT_RANGEDDAMAGE")
    self:UnregisterEvent("PLAYER_DAMAGE_DONE_MODS")
    self:UnregisterEvent("UNIT_ATTACK_SPEED")
    self:UnregisterEvent("UNIT_ATTACK_POWER")
    self:UnregisterEvent("UNIT_RANGED_ATTACK_POWER")
    self:UnregisterEvent("UNIT_ATTACK")
    self:UnregisterEvent("UNIT_RESISTANCES")
    self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	if ( self.frame:IsShown() ) then self.frame:Hide() end
end

function Statline:Refresh()
    local stat1, stat2, stat3
    local num, curTal = GetNumTalentGroups(), GetActiveTalentGroup()
    if ( curTal == 1 ) then
        stat1, stat2, stat3 = GatherStats(db.primary.stat1, db.primary.stat2, db.primary.stat3)
    elseif ( curTal == 2 ) then
        stat1, stat2, stat3 = GatherStats(db.secondary.stat1, db.secondary.stat2, db.secondary.stat3)
    end
    
    if ( SLDataText.db.profile.locked and not db.hideTooltip ) then
        self.frame:EnableMouse(true)
        self.frame:SetScript("OnEnter", function(self)
            GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT")
            GameTooltip:AddLine("|cffffff00Statline|r")
            --GameTooltip:AddLine("|cffffff00" .. standing .."|r")
            --GameTooltip:AddLine(" ", 1, 1, 1)
            --GameTooltip:AddDoubleLine(L["Current Rep:"], format("%.1f%%", curXPPerc), 1, 1, 1, 1, 1, 1)
            --GameTooltip:AddDoubleLine(L["Rep To Go:"], format("%.1f%%", abs(100 - curXPPerc)), 1, 1, 1, 1, 1, 1)
            GameTooltip:Show()
        end)
        self.frame:SetScript("OnLeave", function()
            if ( GameTooltip:IsShown() ) then
                GameTooltip:Hide()
            end
        end)
    else
        self.frame:EnableMouse(false)
        self.frame:SetScript("OnEnter", nil)
        self.frame:SetScript("OnLeave", nil)
    end
    
    -- Update main blocks and strings
    local font, fontH = self.string:GetFont()
    -- Stat 1
    self.string1:SetFont(font, fontH)
    self.string1:SetShadowColor(0, 0, 0, 1)
    self.string1:SetShadowOffset(1.5, -1.5)
    
    self.string1:SetText(stat1)
    self.block1:SetWidth(self.string1:GetWidth())
    self.block1:SetHeight(self.string1:GetHeight())
    self.string1:SetAllPoints(self.block1)
    -- Stat 2
    self.string2:SetFont(font, fontH)
    self.string2:SetShadowColor(0, 0, 0, 1)
    self.string2:SetShadowOffset(1.5, -1.5)
    
    self.string2:SetText(stat2)
    self.block2:SetWidth(self.string2:GetWidth())
    self.block2:SetHeight(self.string2:GetHeight())
    self.string2:SetAllPoints(self.block2)
    -- Stat 3
    self.string3:SetFont(font, fontH)
    self.string3:SetShadowColor(0, 0, 0, 1)
    self.string3:SetShadowOffset(1.5, -1.5)
    
    self.string3:SetText(stat3)
    self.block3:SetWidth(self.string3:GetWidth())
    self.block3:SetHeight(self.string3:GetHeight())
    self.string3:SetAllPoints(self.block3)
    
    -- Align blocks
    self.block1:ClearAllPoints()
    self.block1:SetPoint("LEFT", self.frame, "LEFT", 0, 0)
    self.block2:ClearAllPoints()
    self.block2:SetPoint("LEFT", self.block1, "RIGHT", 2, 0)
    self.block3:ClearAllPoints()
    self.block3:SetPoint("LEFT", self.block2, "RIGHT", 2, 0)
    
    -- And then update the module for refreshing/resizing text/frame
    local color = SLDataText.db.profile.gColor
    local width = (self.string1:GetWidth()+self.string2:GetWidth()+self.string3:GetWidth()+4)
    self.string:SetWidth(width)
    self.string:SetHeight(self.string1:GetHeight())
    
    SLDataText:UpdateModule(self)
end